'use strict';

exports.__esModule = true;
exports.mapValues = mapValues;
exports.createReducer = createReducer;

function defaultEqualityCheck(a, b) {
  return a === b;
}

function defaultMemoize(func) {
  var equalityCheck = arguments.length <= 1 || arguments[1] === undefined ? defaultEqualityCheck : arguments[1];

  var lastArgs = null;
  var lastResult = null;
  return function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    if (lastArgs !== null && args.every(function (value, index) {
      return equalityCheck(value, lastArgs[index]);
    })) {
      return lastResult;
    }
    lastArgs = args;
    lastResult = func.apply(undefined, args);
    return lastResult;
  };
}

function mapValues(obj, fn) {
  return Object.keys(obj).reduce(function (result, key) {
    result[key] = fn(obj[key], key);
    return result;
  }, {});
}

function parseArgs(args) {
  if (args.length == 1) {
    return {
      reducerDependencies: {},
      reducer: args[0],
      reducerMemoizer: defaultMemoize // TODO allow customization
    };
  } else if (args.length == 2) {
      return {
        reducerDependencies: args[0],
        reducer: args[1],
        reducerMemoizer: defaultMemoize
      };
    } else {
      throw new Error('Bad usage'); // TODO better errors
    }
}

function createReducer() {
  var _parseArgs = parseArgs(arguments);

  var reducerDependencies = _parseArgs.reducerDependencies;
  var reducer = _parseArgs.reducer;
  var reducerMemoizer = _parseArgs.reducerMemoizer;

  var reducerStates = mapValues(reducerDependencies, function () {
    return undefined;
  });

  return reducerMemoizer(function finalReducer(state, action) {
    reducerStates = mapValues(reducerStates, function (state, key) {
      return reducerDependencies[key](state, action);
    });
    return reducer(state, action, reducerStates);
  });
}